@@ -2,12 +2,12 @@
 // Program to pull the information out of various types of EXIF digital 
 // camera files and show it in a reasonably consistent way
 //
-// Version 3.00
+// Version 3.03
 //
 // Compiling under Windows:  
 //   Make sure you have Microsoft's compiler on the path, then run make.bat
 //
-// Dec 1999 - Feb 2015
+// Dec 1999 - Dec 2018
 //
 // by Matthias Wandel   www.sentex.net/~mwandel
 //--------------------------------------------------------------------------
@@ -19,7 +19,7 @@
 
 #include <sys/stat.h>
 
-#define JHEAD_VERSION "3.00"
+#define JHEAD_VERSION "3.03"
 
 // This #define turns on features that are too very specific to 
 // how I organize my photos.  Best to ignore everything inside #ifdef MATTHIAS
@@ -59,6 +59,7 @@ static int CreateExifSection = FALSE;
 static int TrimExifTrailingZeroes = FALSE;
 static char * ApplyCommand = NULL;  // Apply this command to all images.
 static char * FilterModel = NULL;
+static int    FilterQuality = 0;
 static int    ExifOnly    = FALSE;
 static int    PortraitOnly = FALSE;
 static time_t ExifTimeAdjust = 0;   // Timezone adjust
@@ -370,11 +371,11 @@ static void DoCommand(const char * FileName, int ShowIt)
     strcpy(TempName+a, "XXXXXX");
 
     // Note: Compiler will warn about mkstemp.  but I need a filename, not a file.
-    // I could just then get the fiel name from what mkstemp made, and pass that
+    // I could just then get the file name from what mkstemp made, and pass that
     // to the executable, but that would make for the exact same vulnerability
     // as mktemp - that is, that between getting the random name, and making the file
     // some other program could snatch that exact same name!
-    // also, not all pltforms support mkstemp.
+    // also, not all platforms support mkstemp.
     mktemp(TempName);
 
 
@@ -457,6 +458,12 @@ static int CheckFileSkip(void)
             return TRUE;
         }
     }
+	if (FilterQuality > 0){
+		//Filter by above threshold quality
+		if (ImageInfo.QualityGuess < FilterQuality){
+			return TRUE;
+		}
+	}
 
     if (ExifOnly){
         // Filtering by EXIF only.  Skip all files that have no Exif.
@@ -711,11 +718,12 @@ static int DoAutoRotate(const char * FileName)
     if (ImageInfo.Orientation != 1){
         const char * Argument;
         Argument = ClearOrientation();
+        if (Argument == NULL) return FALSE; // orientation tag in image, nothing changed.
 
         if (!ZeroRotateTagOnly){
             char RotateCommand[PATH_MAX*2+50];
-            if (Argument == NULL){
-                ErrNonfatal("Unknown orientation tag",0,0);
+            if (strlen(Argument) == 0){
+                // Unknown orientation, but still modified.                
                 return TRUE; // Image is still modified.
             }
             sprintf(RotateCommand, "jpegtran -trim -%s -outfile &o &i", Argument);
@@ -805,6 +813,7 @@ static void ProcessFile(const char * FileName)
     FilesMatched = 1; 
 
     ResetJpgfile();
+    Clear_EXIF();
 
     // Start with an empty image information structure.
     memset(&ImageInfo, 0, sizeof(ImageInfo));
@@ -1250,7 +1259,7 @@ badtime:
 static void Usage (void)
 {
     printf("Jhead is a program for manipulating settings and thumbnails in Exif jpeg headers\n"
-           "used by most Digital Cameras.  v"JHEAD_VERSION" Matthias Wandel, Jan 30 2013.\n"
+           "used by most Digital Cameras.  v"JHEAD_VERSION" Matthias Wandel, Dec 31 2018.\n"
            "http://www.sentex.net/~mwandel/jhead\n"
            "\n");
 
@@ -1322,6 +1331,7 @@ static void Usage (void)
            "             and time in the format yyyy:mm:dd/hh:mm:ss\n"
            "  -ts<time>  Set the Exif internal time to <time>.  <time> is in the format\n"
            "             yyyy:mm:dd-hh:mm:ss\n"
+           "  -tf file   Set the exif time to the modicfation time from another file\n"
            "  -ds<date>  Set the Exif internal date.  <date> is in the format YYYY:MM:DD\n"
            "             or YYYY:MM or YYYY\n"
 
@@ -1342,6 +1352,7 @@ static void Usage (void)
 
            "\nROTATION TAG MANIPULATION:\n"
            "  -autorot   Invoke jpegtran to rotate images according to Exif orientation tag\n"
+           "             anc clear Exif orientation tag\n"
            "             Note: Windows users must get jpegtran for this to work\n"
            "  -norot     Zero out the rotation tag.  This to avoid some browsers from\n" 
            "             rotating the image again after you rotated it but neglected to\n"
@@ -1363,6 +1374,7 @@ static void Usage (void)
            "             camera model description\n"
            "  -exonly    Skip all files that don't have an exif header (skip all jpegs that\n"
            "             were not created by digicam)\n"
+		   "  -quality x Only work on images with JPEG quality factor x or higher\n"
            "  -cmd command\n"
            "             Apply 'command' to every file, then re-insert exif and command\n"
            "             sections into the image. &i will be substituted for the input file\n"
@@ -1595,7 +1607,7 @@ int main (int argc, char **argv)
             ExifTimeAdjust = NewDate-OldDate;
             DoModify |= MODIFY_JPEG;
         }else if (!memcmp(arg,"-dsft",5)){
-            // Set file time to date/time in exif
+            // Set exif time to the timestamp of the file.
             FileTimeToExif = TRUE;
             DoModify |= MODIFY_JPEG;
         }else if (!memcmp(arg,"-ds",3)){
@@ -1640,11 +1652,26 @@ int main (int argc, char **argv)
 
             if ((int)ExifTimeSet == -1) ErrFatal("Time specified is out of range");
             DoModify |= MODIFY_JPEG;
+            
+        }else if (!memcmp(arg,"-tf",3)){
+            // Set the exif time to the modification time from another file.
+            struct stat stat_buf;
+            if (stat(argv[++argn], &stat_buf) == 0){
+                ExifTimeSet = stat_buf.st_mtime;
+            }else{
+                ErrFatal("Could not read file");
+            }
+            DoModify |= MODIFY_JPEG;
 
     // File matching and selection
         }else if (!strcmp(arg,"-model")){
             if (argn+1 >= argc) Usage(); // No extra argument.
             FilterModel = argv[++argn];
+        }else if (!strcmp(arg,"-quality")){
+            if (argn+1 >= argc) Usage(); // No extra argument.
+			if (sscanf(argv[++argn], "%d", &FilterQuality) != 1){
+				Usage();
+			}
         }else if (!strcmp(arg,"-exonly")){
             ExifOnly = 1;
         }else if (!strcmp(arg,"-orp")){
