@@ -289,7 +289,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                     }else if (memcmp(Data+2, "http:", 5) == 0){
                         Sections[SectionsRead-1].Type = M_XMP; // Change tag for internal purposes.
                         if (ShowTags){
-                            printf("Image cotains XMP section, %d bytes long\n", itemlen);
+                            printf("Image contains XMP section, %d bytes long\n", itemlen);
                             if (ShowTags){
                                 ShowXmp(Sections[SectionsRead-1]);
                             }
@@ -304,7 +304,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
             case M_IPTC:
                 if (ReadMode & READ_METADATA){
                     if (ShowTags){
-                        printf("Image cotains IPTC section, %d bytes long\n", itemlen);
+                        printf("Image contains IPTC section, %d bytes long\n", itemlen);
                     }
                     // Note: We just store the IPTC section.  Its relatively straightforward
                     // and we don't act on any part of it, so just display it at parse time.
@@ -454,6 +454,7 @@ int ReplaceThumbnail(const char * ThumbFileName)
         ThumbnailFile = fopen(ThumbFileName,"rb");
 
         if (ThumbnailFile == NULL){
+            noread:
             ErrFatal("Could not read thumbnail file");
             return FALSE;
         }
@@ -484,7 +485,9 @@ int ReplaceThumbnail(const char * ThumbFileName)
     ThumbnailPointer = ExifSection->Data+ImageInfo.ThumbnailOffset+8;
 
     if (ThumbnailFile){
-        fread(ThumbnailPointer, ThumbLen, 1, ThumbnailFile);
+        if (fread(ThumbnailPointer, 1, ThumbLen, ThumbnailFile) != ThumbLen){
+            goto noread;
+        }
         fclose(ThumbnailFile);
     }
 
@@ -649,6 +652,7 @@ Section_t * FindSection(int SectionType)
 int RemoveSectionType(int SectionType)
 {
     int a;
+    int retval = FALSE;
     for (a=0;a<SectionsRead-1;a++){
         if (Sections[a].Type == SectionType){
             // Free up this section
@@ -656,10 +660,11 @@ int RemoveSectionType(int SectionType)
             // Move succeding sections back by one to close space in array.
             memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a));
             SectionsRead -= 1;
-            return TRUE;
+            a -= 1;
+            retval = TRUE;
         }
     }
-    return FALSE;
+    return retval;
 }
 
 //--------------------------------------------------------------------------
